#!/usr/bin/perl -Tw

#######################################################
#
# Family Tree generation program, v2.0
# Written by Ferenc Bodon and Simon Ward, March 2000 (simonward.com)
# Copyright (C) 2000 Ferenc Bodon, Simon K Ward
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# For a copy of the GNU General Public License, visit 
# http://www.gnu.org or write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#######################################################


package PersonPage;
use FamilyTreeBase;
@EXPORT = qw(new main);
use Params::Validate qw(:all);
use strict;
use warnings;
my $q = new CGI;

use base 'FamilyTreeBase';
sub new {
  my $type = shift;
  my $self = $type->SUPER::new(@_);
  $self->{target_person} = undef;
  return $self;
}

sub main {
  my ($self) = validate_pos(@_, {type => HASHREF});
  $self->process_parameters();
  $self->target_check();
  $self->password_check();
  $self->{imgwidth}  = 240;
  $self->{imgheight} = $self->{imgwidth} * 1.5;
  @{ $self->{peers} } = $self->{family_tree_data}->get_peers( $self->{target_person} );
  $self->draw_single_person_page( $self->{target_person} );
  $self->endpage();
}
#######################################################
# processing the parameters (type and passwd)
sub process_parameters {
  my ($self) = validate_pos(@_, {type => HASHREF});
  $self->SUPER::process_parameters();
  my $id = CGI::param("target");
  $self->{target_person} = $self->{family_tree_data}{people}{$id};
}

#######################################################
# check if target person exists in database
sub target_check {
  my ($self) = validate_pos(@_, {type => HASHREF});
  if ( !defined $self->{target_person} ) {
    my $title = $self->{textGenerator}->noDataAbout( CGI::param("target") );
    $self->toppage($title);
    print $q->br, $title, $q->br, "\n";
    $self->endpage();
    exit 1;
  }
}

sub draw_a_person_info {
  my ( $self, $the_person, $cat, $field, $func ) = 
    validate_pos(@_, {type => HASHREF}, {type => HASHREF}, 
    {type => SCALAR|UNDEF}, {type => SCALAR}, {type => CODEREF, optional => 1});
  my $info = defined $cat ? $the_person->{$cat}{$field} : $the_person->{$field};
  if ( defined $info ) {
    if(ref($info) eq "ARRAY" ) {
      if(defined $func) {
        my $new_info = "";
        $new_info .= $func->($_).", " for(@$info);
        $info = $new_info;  
      }
      else {
        $info = join(', ', @$info) ;
      }
    }
    else {
      $info = $func->($info) if(defined $func);
    }
    
    print $q->start_Tr,
      $q->td( $q->strong( $self->{textGenerator}->{$field} ) );
    print $q->td($info) ;      
    print $q->end_Tr, "\n";
  }
}
sub draw_pictures {
  my ( $self, $the_person ) = validate_pos(@_, {type => HASHREF}, {type => HASHREF});
  my $column_number = 3;
  my $index = 0;
  print $q->hr, $q->h3("Image gallery"),
    $q->br, $q->start_table({-align => "center"});
  foreach my $picture (@{$the_person->{pictures}}) {
    print "\n";
    print $q->start_Tr() if ( $index % $column_number == 0 );
    print $q->start_td({-width => 220}),
      $q->a({-href => $picture->{file_name}},
            $q->img({-width => 200,
               -src => $picture->{file_name},
               -alt => $picture->{comment}})), $q->br,
      $picture->{comment}, $q->end_td;
      print $q->end_Tr() if ( ($index % $column_number) == $column_number-1 );
      ++$index;
  } 
  print $q->end_Tr(),"\n" if ( ($index % $column_number) != 0 );
  print $q->br, $q->end_table(), $q->br;
}
sub draw_single_person_page {
  my ( $self, $the_person ) = validate_pos(@_, {type => HASHREF}, {type => HASHREF});
  $self->toppage( $the_person->{name}->long_name() );

  print $q->start_center;
  $self->print_zoom_buttons();
  print $self->html_img($the_person), $q->br, "\n";
  print $the_person->{date_of_birth}
    if ( defined $the_person->{date_of_birth} );
  print " - ";
  print $the_person->{date_of_death}
    if ( defined $the_person->{date_of_death} );
  print $q->end_center, "\n";

  print $q->br, $q->start_table( { -cellspacing => 2, -cellpadding => 2 } );
  $self->draw_a_person_info($the_person, "name", "nickname");
  $self->draw_a_person_info($the_person, undef, "place_of_birth", \&Place::toString);
  $self->draw_a_person_info($the_person, undef, "place_of_death", \&Place::toString);
  $self->draw_a_person_info($the_person, undef, "cemetery", \&Place::toString);
  $self->draw_a_person_info($the_person, undef, "schools" );
  $self->draw_a_person_info($the_person, undef, "jobs" );
  $self->draw_a_person_info($the_person, undef, "work_places" );
  $self->draw_a_person_info($the_person, undef, "places_of_living", \&Place::toString );
  
  if ( defined $the_person->{father} ) {
    print $q->start_Tr,
      $q->td( $q->strong( $self->{textGenerator}->{father} ) ),
      $q->td( $self->aref_tree($the_person->{father}{name}->long_name(), $the_person->{father}, 0)),      
      $q->end_Tr, "\n";
  }
  if ( defined $the_person->{mother} ) {
    print $q->start_Tr,
      $q->td( $q->strong( $self->{textGenerator}->{mother} ) ),
      $q->td( $self->aref_tree($the_person->{mother}{name}->long_name(), $the_person->{mother}, 0)),
      $q->end_Tr, "\n";
  }
  my $spouses_set = $self->{family_tree_data}->get_spouses(
    $self->{target_person} );
  my @spouses_set_as_array = $spouses_set->elements;
  my $spouse_type = $self->{target_person}->{gender} == 0 ? "wives" : "husbands";
  $self->print_people_list( $spouse_type, \@spouses_set_as_array );
  $self->print_people_list( "siblings", $self->{peers} );
  
  my @peers_on_father = $self->{family_tree_data}->get_soft_peers( $self->{target_person}, "father" ); 
  $self->print_people_list( "siblings_on_father", \@peers_on_father );
  my @peers_on_mother = $self->{family_tree_data}->get_soft_peers( $self->{target_person}, "mother" ); 
  $self->print_people_list( "siblings_on_mother", \@peers_on_mother );
  $self->print_people_list( "children",
    $self->{family_tree_data}{children}{ $the_person->{id} } );

  $self->draw_a_person_info($the_person, undef, "email" );
  $self->draw_a_person_info($the_person, undef, "homepage" );
  $self->draw_a_person_info($the_person, undef, "general" );

  print $q->end_table;
#  $self->draw_pictures($the_person)
#    if(defined $the_person->{pictures})
}

sub print_zoom_buttons {
  my ($self)     = validate_pos(@_, {type => HASHREF});
  my $lev_plus1  = 2;

  print $q->start_table(
    { -border => "0", -cellpadding => "0", -cellspacing => "2" } ), "\n",
    $q->start_Tr;
  print $q->start_td({-align => "center"}), "\n",
  $self->aref_tree($q->img( {
          -src => "$self->{graphicsUrl}/zoomout.gif",
          -alt => $self->{textGenerator}->ZoomOut($lev_plus1) }), $self->{target_person}, $lev_plus1),
      $q->end_td, $q->end_Tr, "\n",
      $q->end_table, $q->br, $q->br, "\n";
}

sub print_people_list {
  my ( $self, $people_type, $people_array_r ) = validate_pos(@_, 
    {type => HASHREF}, {type => SCALAR}, {type => ARRAYREF});
  if ( defined $people_array_r && @$people_array_r > 0) {
    print $q->start_Tr,
      $q->td( $q->strong( $self->{textGenerator}->{$people_type} ) ),
      $q->start_td;

    foreach my $person (@$people_array_r) {
      if ( $person != $self->{target_person} && $person != $self->{NONE_person}) {
        print $self->aref_tree($person->{name}->full_name(), $person, 0), ", ";
      }
    }
    print $q->end_td, $q->end_Tr;
  }
}


1;

